; An example of a simple combo box (style=CBS_SIMPLE) with the edit box portion subclassed so that
;   the user can type in his own selection and hit return to accept it, or hit escape to abort.

.686p
.model  flat, stdcall
option  casemap :none   ; case sensitive
.nolist
include windows.inc

; some miscellaneous macros
soff Macro QuotedText:Vararg    ; returns offset to a string
Local LocalText
.data
LocalText db QuotedText,0
.code
EXITM <offset LocalText>
Endm

GetHandle Macro MyId:Req,MyHandle:Req 
    invoke GetDlgItem,hWin,MyId
    mov MyHandle,eax         ; save handle to control
endm

uselib  MACRO   libname
    include     libname.inc
    includelib  libname.lib
ENDM

uselib user32
uselib kernel32
uselib comctl32

;uselib shell32
;uselib comdlg32
;uselib gdi32
;uselib masm32   ; for debug
;uselib debug    ; for debug

.listall
inv equ invoke

DlgProc     PROTO :DWORD,:DWORD,:DWORD,:DWORD

cb equ 1041     ; id of combobox

.data?
hInst dd ?
hWin  dd ?
hcb   dd ?      ; handle of combobox

hComboEdit dd ?     ; edit part of combo box
OldComboEdit dd ?   ; address of combobox edit before subclass

buff  db 100 dup (?)    ; scratch

COMBOBOXINFO STRUCT
  cbSize            DWORD  ?
  rcItem            RECT  <>
  rcButton          RECT  <>
  stateButton       DWORD  ?
  hwndCombo         DWORD  ?
  hwndItem          DWORD  ?
  hwndList          DWORD  ?
COMBOBOXINFO ENDS

ci COMBOBOXINFO <?>

.code
Program:
    invoke  GetModuleHandle, NULL
    mov hInst,eax
    invoke  InitCommonControls
    invoke  DialogBoxParam, hInst, 101, 0, ADDR DlgProc, 0
    invoke  ExitProcess, eax

DlgProc proc hWnd:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
    .if uMsg == WM_COMMAND
        mov eax,wParam  ; identifier of the control, or accelerator
        mov edx,eax     ; check hiword (notification code)
        shr edx,16
        .if lParam==0   ; menu item or accelerator if zero, else a control
            ; etc
        .else   ; a control
            .if ax==cb  ; our combo box
                .if dx==CBN_DBLCLK          ; user double clicked or hit enter key.
                    call ComboGetText
                .elseif dx==CBN_KILLFOCUS   ; don't seem to get this with the combobox edit subclassed?
                    ;Normally, you would close the combobox here 
                .elseif dx==CBN_SELENDCANCEL ; this is not normally sent for a CBS_SIMPLE style combobox, 
                                             ;  we faked it in EditWndProc so user could abort by hitting escape key. 
                    invoke MessageBox,0,soff("combo box selection aborted, would normally just close the combobox here"),0,0
                .endif
            .endif
        .endif
    .elseif uMsg == WM_INITDIALOG
        mov eax,hWnd    ; save our handle
        mov hWin,eax
        GetHandle cb,hcb    ; the the handle to the combobox
        mov ci.cbSize,sizeof(COMBOBOXINFO)  ; now subclass the edit box within the combobox
        inv GetComboBoxInfo,hcb,addr ci
        mov eax,ci.hwndItem                 ; want the edit box
        mov hComboEdit,eax                      ; save
        inv SetWindowLong,hComboEdit,GWL_WNDPROC,EditWndProc
        mov OldComboEdit,eax                ; save old address for later

        call FillCombo      ; put some data in for our tests
        invoke SetFocus,hComboEdit  ;normally we would fill on demand but this is just a test
    .elseif uMsg == WM_CLOSE
        invoke  EndDialog,hWin,0
    .endif

    xor eax,eax
    ret
DlgProc endp

EditWndProc PROC hEdit:DWord,uMsg:DWord,wParam:DWord,lParam:DWord
    ;traps messages to the Combobox to catch enter key
    .if uMsg==WM_KEYDOWN
        mov eax,hEdit
        .if eax==hComboEdit    ; that's us
            mov eax,wParam
            .if eax==VK_RETURN  ; treat as double click
                inv PostMessage,hWin,WM_COMMAND,(CBN_DBLCLK shl 16) or cb,hcb
            .elseif eax==VK_ESCAPE  ; user wants to quit
                inv PostMessage,hWin,WM_COMMAND,(CBN_SELENDCANCEL shl 16) or cb,hcb
            .endif
        .endif
    .elseif uMsg == WM_GETDLGCODE
        mov eax,DLGC_WANTALLKEYS    ; need this to get enter key on WM_KEYDOWN
        ret
    .endif
    inv CallWindowProc,OldComboEdit,hEdit,uMsg,wParam,lParam    
    ret
EditWndProc Endp

ComboGetText proc   ; retrieve the users entry and put it in an edit box for a test
    inv SendMessage,hcb,WM_GETTEXT,100,addr buff
    invoke GetDlgItem,hWin,1004
    invoke SendMessage,eax,WM_SETTEXT,0,addr buff
    ret
ComboGetText EndP

FillCombo proc  ; add test junk lines
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("aline 1") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("bline 2") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("cline 3") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("dline 4") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("eline 5") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("fline 6") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("gline 7") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("hline 8") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("iline 9") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("jline 10") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("kline 11") 
    inv SendMessage,hcb,CB_ADDSTRING,0,soff("lline 12") 
    ret
FillCombo EndP
end Program